/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;


import java.awt.Color;
import java.util.Vector;

/**
 * Class which represents a container for a config element
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class ConfigElement {
	private String effectName_ = "";
	private String displayName_ = "";
	private Vector<Attribute> attributes_ = null;
	private Attribute tmpAttribute = null;
	private Vector<String[]> tmpEnumeration = null;
	private Color effectColor = null;
	private Color selectColor = null;

	/**
	 * Constructor
	 */
	public ConfigElement() {
		attributes_ = new Vector<Attribute>();
	}
	
	/**
	 * Method for retrieving the effect name
	 * @return String with name
	 */
	public String getEffectName() {
		return effectName_;
	}
	
	/**
	 * Method for setting the effect name
	 * @param name String with name
	 */
	public void setEffectName(String name) {
		effectName_ = name;
	}
	
	/**
	 * Method for retrieving the display name
	 * @return String with name
	 */
	public String getDisplayName() {
		return displayName_;
	}
	
	/**
	 * Method for setting the display name
	 * @param name String with name
	 */
	public void setDisplayName(String name) {
		displayName_ = name;
	}
	
	/**
	 * Method for retrieving the effect color
	 * @return Color with effect color
	 */
	public Color getEffectColor() {
		return effectColor;
	}

	/**
	 * Method for setting the effect color
	 * @param effectColor Color with effect color
	 */
	public void setEffectColor(Color effectColor) {
		this.effectColor = effectColor;
	}
	
	/**
	 * Method for retrieving the select color
	 * @return Color with the select color
	 */
	public Color getSelectColor() {
		return selectColor;
	}

	/**
	 * Method for setting the select color
	 * @param selectColor Color with the select color
	 */
	public void setSelectColor(Color selectColor) {
		this.selectColor = selectColor;
	}

	/**
	 * Method for retrieving the attributes
	 * @return Vector<Attribute> with attributes
	 */
	public Vector<Attribute> getAttributes() {
		return attributes_;
	}
	
	/**
	 * Method for opening a new attribute
	 */
	public void openNewAttribute() {
		tmpAttribute = new Attribute();
	}
	
	/**
	 * Method for opening a new enumeration
	 */
	public void openNewEnumeration() {
		tmpEnumeration = new Vector<String[]>();
	}
	
	/**
	 * Method for closing an attribute
	 */
	public void closeAttribute() {
		attributes_.add(tmpAttribute);
		tmpAttribute = null;
	}
	
	/**
	 * Method for closing an enumeration
	 */
	public void closeEnumeration() {
		tmpAttribute.addEnumeration(tmpEnumeration);
		tmpEnumeration = null;
	}
	
	/**
	 * Method for adding an attribute
	 * @param attribute String[] with attribute (0 = attribute name, 1 = attribute value)
	 * @return true if added successfully else false
	 */
	public boolean addAttribute(String[] attribute) {
		if (attribute == null)
			return false;
	
		tmpAttribute.addAttribute(attribute);
		return true;
	}
	
	/**
	 * Method for adding an enumeration
	 * @param enumeration String[] with enumeration (0 = attribute name, 1 = attribute value)
	 * @return true if added successfully else false
	 */
	public boolean addEnumeration(String[] enumeration) {
		if (enumeration == null)
			return false;
	
		tmpEnumeration.add(enumeration);
		return true;
	}
	
	/**
	 * Method for adding a complete set of attributes
	 * @param attributes Vector<String[]> with all attributes
	 * @param enumeration Vector<String[]> with all enumerations
	 * @return true if added successfully else false
	 */
	public boolean addCompleteAttribute(Vector<String[]> attributes, Vector<String[]> enumeration) {
		if (attributes == null || attributes.isEmpty())
			return false;
	
		Attribute attr = new Attribute(attributes, enumeration);		
		attributes_.add(attr);
		return true;
	}
	
	/**
	 * Method for getting a string representation of the effect element
	 * @param String with representation
	 */
	public String toString() {
		String s = "Name: " + effectName_ + "\n";
		s += "Display Name: " + displayName_ + "\n";
		
		s += "Attributes:\n";
		for (int i = 0; i < attributes_.size(); i++) {
			Attribute v = attributes_.elementAt(i);
			s += v;			
		}
		
		s += "\n";
		
		return s;
	}
}
