/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */

package lib;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import lib.ColorRectangle;
import lib.BarElement;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.Collections;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.plaf.basic.BasicComboBoxEditor;
import javax.swing.text.MaskFormatter;
import javax.swing.JSeparator;

import jaxb.configuration.Types;

// TODO: resizing with mouse

/**
 * Class which represents an effect panel with mouse drawing support
 * 
 * @author Markus Waltl
 * @version 1.0
 */
public class EffectPanel extends JPanel implements KeyListener, ActionListener, MouseListener, MouseMotionListener {
	private static final long serialVersionUID = -8255392254901782545L;
	private String nameOfEffect = "";
	private Vector<BarElement> bar = null;
	private EffectElement effectElement = null;	
	private ConfigElement configElement = null;
	private JLabel lblMessage = new JLabel();
	private String editingRect = "";
	private JFormattedTextField fieldStartTime = null;
	private JFormattedTextField fieldEndTime = null;	
	private int startX = 0;
	private boolean isInit = false;
	public final int HEIGHT = 19;
	public final int POSY = 0;
	private int prevPosX = 0;
	private Color color = null;
	private Color selColor = null;
	private int curWidth = 0;
	private int curStartX = 0;
	private JPopupMenu contextMenu = null;
	private JMenuItem settingsItem = null;
	private JMenuItem deleteItem = null;	
	private int mouseContextPosX = -1;
	private int posX = -1;
	private ColorRectangle selectedRect = null;
	private int oldDragPosX = -1;
	private boolean isInputOK = true;
	private boolean leftMouseButtonPressed = false;
	private JButton buttonStartTime = null;
	private JButton buttonEndTime = null;
	
	private final static String TAB = "        "; // 8 spaces corresponding to default tab size in Java
	
	/**
	 * Constructor
	 * @param color Color of the bar
	 * @param selColor Color of the selected bar
	 * @param ce ConfigElement for the bar
	 */
	public EffectPanel(Color color, Color selColor, ConfigElement ce) {
		if (color == null || ce == null || selColor == null) {
			System.out.println("No color/selection color or config element set!");
			return;
		}
		
		this.color = color;
		this.selColor = selColor;
		effectElement = new EffectElement(ce);
		configElement = ce; // only needed for loadSEM
		bar = new Vector<BarElement>();
		contextMenu = new JPopupMenu();
		settingsItem = new JMenuItem("Settings");
		settingsItem.addActionListener(this);
		settingsItem.setActionCommand("contextSettings");
		settingsItem.setName("contextSettings");
		deleteItem = new JMenuItem("Delete");
		deleteItem.addActionListener(this);
		deleteItem.setActionCommand("contextDelete");
		deleteItem.setName("contextDelete");
		contextMenu.add(deleteItem);
		contextMenu.add(new JSeparator());
		contextMenu.add(settingsItem);
		buttonStartTime = new JButton("Set 0");
		buttonStartTime.addActionListener(this);
		buttonStartTime.setActionCommand("setToStartTime");
		buttonEndTime = new JButton("Set L");
		buttonEndTime.addActionListener(this);
		buttonEndTime.setActionCommand("setToEndTime");
		isInit = false;
		addMouseListener(this);
		addKeyListener(this);
		addMouseMotionListener(this);
	}
	
	public void paint(Graphics g) {
		super.paint( g );
		
		ColorRectangle curRect = null;
		int testPosX = 0;
		if (curStartX == startX)
			testPosX = startX;
		else
			testPosX = curStartX-(int)Math.round((curStartX-posX)/7.0)*7;
		
		int testWidth = 0;
		if (curWidth != 0) {
			if (posX > startX)
				testWidth = curWidth+(int)Math.round((posX-prevPosX)/7.0)*7;
			else
				testWidth = curWidth+(int)Math.round((curStartX-posX)/7.0)*7;
		}
						
		curRect = new ColorRectangle(testPosX, POSY, testWidth, HEIGHT, "newRect");
		
		// check for borders
		if (testPosX < 0) {
			testPosX = 0;
			posX = 0;
		}
		
		if ((testPosX+testWidth) > this.getWidth()) {
			testWidth = this.getWidth();
			posX = this.getWidth();
		}
		
		int xToFill = -1;
		int endXtoFill = -1;
		boolean overlapping = false;
		if (bar != null) {
			if (posX > startX)
				Collections.sort(bar);
			else
				Collections.sort(bar, Collections.reverseOrder());
			
			Iterator<BarElement> it = bar.iterator();
			while (it.hasNext()) {
				BarElement be = (BarElement) it.next();
				ColorRectangle key = be.getColorRectangle();
				g.setColor(key.getColor());
				g.fillRect(key.x, key.y, key.width, key.height);
				g.setColor(selColor);
				g.drawRect(key.x, key.y, key.width, key.height);
				
				if (!overlapping && key.intersects(curRect)) {
					overlapping = true;
					xToFill = key.x;
					endXtoFill = key.x+key.width;
				}
			}
		}
		
		if (color != null)
			g.setColor(color);
		
		if (!overlapping && posX > -1) {
			if (posX > startX) {
				if (posX > prevPosX) {
					g.clearRect(startX, POSY, posX-startX, HEIGHT);
					g.fillRect(startX, POSY, posX-startX, HEIGHT);	
				}
				else {
					g.clearRect(startX, POSY, prevPosX-startX, HEIGHT);
					g.fillRect(startX, POSY, posX-startX, HEIGHT);
				}
				prevPosX = posX;
				curWidth = posX-startX;
				curStartX = startX;
			}
			else if (posX < startX) {
				if (posX > prevPosX) {
					g.clearRect(posX, POSY, startX-posX, HEIGHT);
					g.fillRect(posX, POSY, startX-posX, HEIGHT);
				}
				else {
					g.clearRect(posX, POSY, prevPosX-posX, HEIGHT);
					g.fillRect(posX, POSY, startX-posX, HEIGHT);
				}
				prevPosX = startX;
				curWidth = startX-posX;
				curStartX = posX;
			}
			else
				curWidth = 0;
		}
		else if (overlapping && xToFill > -1) {
			if (posX > startX) {
				g.clearRect(startX, POSY, xToFill-startX, HEIGHT);
				g.fillRect(startX, POSY, xToFill-startX, HEIGHT);
				curStartX = startX;
				curWidth = xToFill-startX;
			}
			else if (posX < startX) {
				g.clearRect(endXtoFill, POSY, startX-endXtoFill, HEIGHT);
				g.fillRect(endXtoFill, POSY, startX-endXtoFill, HEIGHT);
				curStartX = endXtoFill;
				curWidth = startX-endXtoFill;
			}
		}
	}
	
	/**
	 * Method for drawing the color rectangle
	 * @param posX int with position on the x-axes
	 */
	public void drawColorRect(int posX) {
		if (isInit) {
			this.posX = (int)Math.round(posX/7.0)*7;
			repaint();
		}
	}
	
	/**
	 * Method for initializing the color rectangle
	 * @param posX int with position on the x-axes
	 */
	public void initColorRect(int posX) {
		if (selectedRect != null) {
			if (selectedRect.contains(posX, HEIGHT/2))
				return;
			else {
				boolean found = false;
				for (int i = 0; i < bar.size() && !found; i++) {
					BarElement be = bar.elementAt(i);
					ColorRectangle rc = be.getColorRectangle();
					if (rc.getIdentifier().equals(selectedRect.getIdentifier())) {
						rc.setColor(color);
						found = true;
					}
				}
				selectedRect = null;
			}
		}
		
		startX = (int)Math.round(posX/7.0)*7;
		isInit = true;
	}
	
	/**
	 * Method for adding the color rectangle
	 */
	public void addColorRect() {
		if (selectedRect == null) {
			boolean overlapped = false;
			for (int i = 0; i < bar.size() && !overlapped; i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.intersects(new ColorRectangle(curStartX, POSY, curWidth, HEIGHT, "testRect"))) {
					overlapped = true;
				}
			}
			
			if (!overlapped) {
				// every 7 pixel is 0.1 second
				double startTime = curStartX/70.0; // get seconds			
				DecimalFormat df2 = new DecimalFormat("#########0.0");
				String value = df2.format(startTime);
				value = value.replace(",", ".");
				startTime = new Double(value).doubleValue();
						
				double endTime = (curStartX+curWidth)/70.0; // get seconds	
				value = df2.format(endTime);
				value = value.replace(",", ".");
				endTime = new Double(value).doubleValue();	
				
				if (startTime != endTime && startTime >= 0 && (endTime*70) <= this.getWidth()) {
					// show display for settings!!!
					EffectElement settings = showEffectSettingsDialog(null, effectElement, startTime, endTime);
					if (settings != null && settings.getStartTime() < settings.getEndTime() &&
							settings.getStartTime() >= 0 && (settings.getEndTime()*70) <= this.getWidth())
						bar.add(new BarElement(new ColorRectangle((int)(settings.getStartTime()*70), POSY, (int)((settings.getEndTime()-settings.getStartTime())*70), 
					 			HEIGHT, color, "rect" + bar.size()), settings));
					else {
						if (settings != null)
							System.out.println("removed invalid bar: start: " + settings.getStartTime() + ", end: " + settings.getEndTime());
						
						this.getGraphics().clearRect(curStartX, POSY, curWidth, HEIGHT);
					}
					
					effectElement = new EffectElement(effectElement.getConfigElement());
				}
			}
		}
		
		startX = 0;
		prevPosX = 0;
		curWidth = 0;
		curStartX = 0;
		posX = -1;
		isInit = false;
		repaint();
	}

	public String toString() {
		String s = "";
		for (int i = 0; i < bar.size(); i++) {
			BarElement be = bar.elementAt(i);
			ColorRectangle key = be.getColorRectangle();
			s += "Rect: " + key + "\n";			
			s += be.getEffectElement();					
			s += "\n";
		}
		
		return s;
	}
	
	/**
	 * Method for showing the effect settings dialog
	 * @param parent JFrame with the parent
	 * @param ee EffectElement to be filled
	 * @return EffectElement the with filled effect element
	 */
	public EffectElement showEffectSettingsDialog(JFrame parent, EffectElement ee) {
		return showEffectSettingsDialog(parent, ee, ee.getStartTime(), ee.getEndTime());
	}
	
	/**
	 * Method for showing the effect settings dialog
	 * @param parent JFrame with the parent
	 * @param ee EffectElement to be filled
	 * @param startTime double with the start time
	 * @param endTime double with the end time
	 * @return EffectElement with the filled effect element
	 */
	public EffectElement showEffectSettingsDialog(JFrame parent, EffectElement ee, double startTime, double endTime) {
		if (ee == null || ee.getConfigElement() == null)
			return null;
		
		Vector<Attribute> att = ee.getConfigElement().getAttributes();
		String[] settingTypes = new String[att.size()];
		
		final JPanel content = new JPanel();
		// added 3 for start- and end time and message label
		content.setBounds(0, 0, 300, (att.size()+3)*20);
		content.setPreferredSize(new Dimension(300, (att.size()+3)*20));
		content.setLayout(new GridLayout(0, 2));
		
		for (int i = 0; i < att.size(); i++) {
			// attribute element
			Attribute attr = att.elementAt(i);
			Vector<String[]> v = attr.getAttributes();
			
			JComponent[] row = new JComponent[2];
			String name = "";
			String toolTip = "";
			
			for (int j = 0; j < v.size(); j++) {
				// attributes of element
				String[] attribute = v.elementAt(j);
				if (attribute[0].toLowerCase().equals(ConfigDefinitions.NAME)) {
					JLabel label = new JLabel(attribute[1]);
					row[0] = label;
					name = attribute[1];
					settingTypes[i] = name;
				}
				else if (attribute[0].toLowerCase().equals(ConfigDefinitions.TOOLTIP)) {
					toolTip = attribute[1];
				}
				else if (attribute[0].toLowerCase().equals(ConfigDefinitions.TYPE)) {
					if (attribute[1].toLowerCase().equals(Types.INT.toString().toLowerCase())) {
						IntField fieldInt = new IntField();
						fieldInt.setPreferredSize(new Dimension(80, 20));
						row[1] = fieldInt;
					}
					else if (attribute[1].toLowerCase().equals(Types.STRING.toString().toLowerCase())) {
						JTextField fieldString = new JTextField();
						fieldString.setPreferredSize(new Dimension(150, 20));
						row[1] = fieldString;
					}
					else if (attribute[1].toLowerCase().equals(Types.PERCENT.toString().toLowerCase())) {
						JSpinner spinnerPercent = new JSpinner();
						spinnerPercent.setPreferredSize(new Dimension(50, 20));
						spinnerPercent.setModel(new SpinnerNumberModel(-1, -1, 100, 1));
						row[1] = spinnerPercent;
					}
					else if (attribute[1].toLowerCase().equals(Types.BOOL.toString().toLowerCase())) {
						JCheckBox checkBox = new JCheckBox();
						checkBox.setPreferredSize(new Dimension(50,20));
						row[1] = checkBox;
					}
					else if (attribute[1].toLowerCase().equals(Types.FLOAT.toString().toLowerCase())) {
						FloatField fieldFloat = new FloatField();
						fieldFloat.setPreferredSize(new Dimension(80, 20));
						row[1] = fieldFloat;
					}
					else if (attribute[1].toLowerCase().equals(Types.COLOR.toString().toLowerCase())) {
						JComboBox colorEnum = new JComboBox();
						colorEnum.setRenderer(new ColorCellRenderer(colorEnum));
						colorEnum.setEditable(true);
						colorEnum.setPreferredSize(new Dimension(150, 20));
						colorEnum.setActionCommand("colorSelector");
						colorEnum.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								if (e.getActionCommand().equals("colorSelector")) {
									JComboBox source = (JComboBox)e.getSource();
									if (source.getSelectedItem() != null && source.getSelectedItem() instanceof ComboBoxColorItem) {
										Color newColor = source.getBackground();
										if (((ComboBoxColorItem)source.getSelectedItem()).getText().equals("Open color selector")) {
											newColor = JColorChooser.showDialog(
								                     EffectPanel.this,
								                     "Choose a color",null);
											
											if (newColor != null) {
												ComboBoxColorItem cbci = new ComboBoxColorItem();
												cbci.setText("#" + HelperFunctions.rgbToHex(newColor));
												cbci.setColor(newColor);
		
												boolean found = false;
												for (int i = 0; i < source.getItemCount() && !found; i++) {
													Object o = source.getItemAt(i);
													if (o != null && o instanceof ComboBoxColorItem) {
														ComboBoxColorItem cbciList = (ComboBoxColorItem)o;
														if (cbciList.getText().equals(cbci.getText()))
															found = true;
													}
												}
												
												// add only new color if not already in the list
												if (!found) {
													source.addItem(cbci);
													source.setSelectedItem(cbci);
												}
											}
										}
										else {
											newColor = HelperFunctions.hexToRGB(((ComboBoxColorItem)source.getSelectedItem()).getText());
										}
										
										source.setEditor(new MyComboBoxEditor(newColor));
									}
									else if (source.getSelectedItem() != null && source.getSelectedItem() instanceof String) {
										// add color if not existing after enter
										// catch enter on panel!!!!
										ComboBoxColorItem cbci = new ComboBoxColorItem();
										cbci.setText((String)source.getSelectedItem());
										cbci.setColor(HelperFunctions.hexToRGB((String)source.getSelectedItem()));
										
										boolean found = false;
										for (int i = 0; i < source.getItemCount() && !found; i++) {
											Object o = source.getItemAt(i);
											if (o != null && o instanceof ComboBoxColorItem) {
												ComboBoxColorItem cbciList = (ComboBoxColorItem)o;
												if (cbciList.getText().equals(cbci.getText()))
													found = true;
											}
										}
										
										// add only new color if not already in the list
										if (!found) {
											source.addItem(cbci);
											source.setSelectedItem(cbci);
										}
										
										source.setEditor(new MyComboBoxEditor(HelperFunctions.hexToRGB((String)source.getSelectedItem())));
									}
								}								
							}
						});
						
						// iterate through enumeration parts
						Vector<Vector<String[]>> enumeration = attr.getEnumeration();
						for (int k = 0; k < enumeration.size(); k++) {
							Vector<String[]> v2 = enumeration.elementAt(k);
							ComboBoxColorItem cbci = new ComboBoxColorItem();
							String comboContent1 = "";
							String comboContent2 = "";
							for (int l = 0; l < v2.size(); l++) {
								String[] enumField = v2.elementAt(l);
								if (enumField[0].toLowerCase().equals(ConfigDefinitions.DISPLAY))
									comboContent1 = enumField[1];
								else if (enumField[0].toLowerCase().equals(ConfigDefinitions.STANDARD))
									comboContent2 = enumField[1];
							}
							
							if (!comboContent1.equals("") || !comboContent2.equals("")) {								
								if (!comboContent1.equals("") && !comboContent2.equals("")) {
									if (!comboContent1.equalsIgnoreCase(comboContent2)) {
										cbci.setColor(HelperFunctions.hexToRGB(comboContent1));
										cbci.setText(comboContent1);
										cbci.setStandardUrn(comboContent2);
										colorEnum.addItem(cbci);
									}
									else {
										cbci.setColor(HelperFunctions.hexToRGB(comboContent1));
										cbci.setText(comboContent1);
										cbci.setStandardUrn(comboContent2);
										colorEnum.addItem(cbci);
									}
								}
								else if (!comboContent1.equals("") && comboContent2.equals("")) {
									cbci.setColor(HelperFunctions.hexToRGB(comboContent1));
									cbci.setText(comboContent1);
									cbci.setStandardUrn(comboContent1);
									colorEnum.addItem(cbci);									
								}
								else if (comboContent1.equals("") && !comboContent2.equals("")) {
									cbci.setColor(HelperFunctions.hexToRGB(comboContent2));
									cbci.setText(comboContent2);
									cbci.setStandardUrn(comboContent2);
									colorEnum.addItem(cbci);
								}
							}							
						}
						
						if (colorEnum.getItemCount() > 0) {
							ComboBoxColorItem cbci = new ComboBoxColorItem();
							cbci.setText("Open color selector");
							colorEnum.insertItemAt(cbci, 1); // always be item number 2
						}
						else {
							ComboBoxColorItem cbci = new ComboBoxColorItem();
							cbci.setText("Open color selector");
							colorEnum.addItem(cbci);
						}
						
						row[1] = colorEnum;
					}
					else if (attribute[1].toLowerCase().equals(Types.ENUM.toString().toLowerCase())) {
						JComboBox comboEnum = new JComboBox();
						comboEnum.setPreferredSize(new Dimension(150, 20));
						
						// iterate through enumeration parts
						Vector<Vector<String[]>> enumeration = attr.getEnumeration();
						for (int k = 0; k < enumeration.size(); k++) {
							Vector<String[]> v2 = enumeration.elementAt(k);
							String comboContent1 = "";
							String comboContent2 = "";
							for (int l = 0; l < v2.size(); l++) {
								String[] enumField = v2.elementAt(l);
								if (enumField[0].toLowerCase().equals(ConfigDefinitions.DISPLAY))
									comboContent1 = enumField[1];
								else if (enumField[0].toLowerCase().equals(ConfigDefinitions.STANDARD))
									comboContent2 = enumField[1];
							}
							
							if (!comboContent1.equals("") || !comboContent2.equals("")) {
								if (!comboContent1.equals("") && !comboContent2.equals("")) {
									if (!comboContent1.equalsIgnoreCase(comboContent2))
										// not a very beautiful solution! maybe correct later on for handling standard and display parameter
										comboEnum.addItem(comboContent1 + TAB + TAB + TAB + TAB + TAB + 
												TAB + TAB + TAB + " - " + comboContent2);
									else
										comboEnum.addItem(comboContent1);
								}
								else if (!comboContent1.equals("") && comboContent2.equals(""))
									comboEnum.addItem(comboContent1);
								else if (comboContent1.equals("") && !comboContent2.equals(""))
									comboEnum.addItem(comboContent2);
							}							
						}
						
						row[1] = comboEnum;
					}
					else if (attribute[1].toLowerCase().equals(Types.TIME.toString().toLowerCase())) {
						TimeField fieldTime;
						try {
							fieldTime = new TimeField();
							fieldTime.setPreferredSize(new Dimension(85, 20));
							row[1] = fieldTime;
						} catch (ParseException e) {							
							e.printStackTrace();
						}						
					}
					else {
						System.out.println("there is an unsupported type in the config! (Supported: int, bool, float, string, percent, enum, color, time)");
						return null;
					}
				}
			}
	
			if (!name.equals("")) {
				row[1].setName(name);
			}
			else {
				System.out.println("the name is missing for a field!");
				return null;
			}
			
			// add tooltips to the fields
			row[0].setToolTipText(toolTip);
			row[1].setToolTipText(toolTip);
			
			content.add(row[0]);
			content.add(row[1]);
		}
		
		//if attributes already existing fill fields
		if (ee.getAttributes() != null && !ee.getAttributes().isEmpty()) {
			Component[] comps = content.getComponents();
			Vector<String[]> attributes = ee.getAttributes();
			
			for (int i = 0; i < attributes.size(); i++) {
				String[] attr = attributes.elementAt(i);
				
				for (int j = 0; j < comps.length; j++) {
					Component comp = comps[j];
					
					if (comp.getName() != null && comp.getName().equals(attr[0])){
						if (comp instanceof JSpinner)
							((JSpinner)comp).setValue(Integer.valueOf(attr[1]));
						else if (comp instanceof IntField)
							((IntField)comp).setText(attr[1]);
						else if (comp instanceof FloatField)
							((FloatField)comp).setText(attr[1]);
						else if (comp instanceof JComboBox) {
							boolean found = false;
							for (int k = 0; k < ((JComboBox)comp).getItemCount() && !found; k++) {
								if (((JComboBox)comp).getItemAt(k) instanceof ComboBoxColorItem) {
									ComboBoxColorItem cbci = (ComboBoxColorItem)((JComboBox)comp).getItemAt(k);
									
									if (cbci.getStandardUrn() != null && !cbci.getStandardUrn().equals("")) {
										if (cbci.getStandardUrn().contains(attr[1])) {
											((JComboBox)comp).setSelectedIndex(k);
											found = true;
										}
										else if (cbci.getText().contains(attr[1])) {
											((JComboBox)comp).setSelectedIndex(k);
											found = true;
										}
									}
									else {
										if (cbci.getText().contains(attr[1])) {
											((JComboBox)comp).setSelectedIndex(k);
											found = true;
										}	
									}
								}
								else {
									String item = (String)((JComboBox)comp).getItemAt(k);
									int pos = -1;
									if ((pos = item.indexOf("-")) > -1) {
										item = item.substring(pos+1);
										item = item.trim();
									}
									
									if (item.contains(attr[1])) {
										((JComboBox)comp).setSelectedIndex(k);
										found = true;
									}
								}
							}
							
							if (!found) {
								if (comp instanceof JComboBox && ((JComboBox)comp).getItemAt(0) instanceof ComboBoxColorItem) {
									ComboBoxColorItem cbci = new ComboBoxColorItem();
									cbci.setText(attr[1]);
									cbci.setColor(HelperFunctions.hexToRGB(attr[1]));
									((JComboBox)comp).addItem(cbci);
									((JComboBox)comp).setSelectedItem(cbci);
								}
							}
						}
						else if (comp instanceof JCheckBox) {
							if (attr[1].equals("true") || attr[1].equals("1"))
								((JCheckBox)comp).setSelected(true);
							else
								((JCheckBox)comp).setSelected(false);
						}
						else if (comp instanceof TimeField)	{
							String timeString = HelperFunctions.doubleToTimeString(Double.valueOf(attr[1]));
							((TimeField)comp).setText(timeString);
						}
						else if (comp instanceof JTextField)
							((JTextField)comp).setText(attr[1]);
					}
				}
			}
		}
		
		JLabel sTime = new JLabel("Start time:");
		JLabel eTime = new JLabel("End time:");
		
		// calculate start and end time to string representation
		String strStartTime = HelperFunctions.doubleToTimeString(startTime);
		String strEndTime = HelperFunctions.doubleToTimeString(endTime);
		
		try {
			fieldStartTime = new JFormattedTextField(new MaskFormatter("##:##:##.#"));
			fieldStartTime.setPreferredSize(new Dimension(85, 20));
			fieldStartTime.setText(strStartTime);
			fieldStartTime.setName("startTime");
			fieldStartTime.addKeyListener(this);
			
			fieldEndTime = new JFormattedTextField(new MaskFormatter("##:##:##.#"));
			fieldEndTime.setPreferredSize(new Dimension(85, 20));
			fieldEndTime.setText(strEndTime);
			fieldEndTime.setName("endTime");
			fieldEndTime.addKeyListener(this);
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}	
		
		
		final JPanel startPanel = new JPanel();
		startPanel.setBounds(0, 0, 150, 20);
		startPanel.setPreferredSize(new Dimension(150, 20));
		startPanel.setLayout(new GridLayout(0, 2));
		
		content.add(sTime);
		startPanel.add(fieldStartTime);		
		startPanel.add(buttonStartTime);
		content.add(startPanel);
		
		final JPanel endPanel = new JPanel();
		endPanel.setBounds(0, 0, 150, 20);
		endPanel.setPreferredSize(new Dimension(150, 20));
		endPanel.setLayout(new GridLayout(0, 2));
		
		content.add(eTime);
		endPanel.add(fieldEndTime);		
		endPanel.add(buttonEndTime);
		content.add(endPanel);
		content.add(lblMessage);
				
		boolean accepted = false;
		boolean canceled = false;
		do {
			int retVal = JOptionPane.showConfirmDialog(parent, content, "Enter effect settings", JOptionPane.OK_CANCEL_OPTION);
			if (retVal == JOptionPane.OK_OPTION) {
				if (!isInputOK) {
					retVal = JOptionPane.showConfirmDialog(parent, 
							"Do you really want to use incorrect values? (Note: incorrect time leads to deletion of element or resetting of values!)", "Incorrect values", JOptionPane.OK_CANCEL_OPTION);
					if (retVal == JOptionPane.OK_OPTION)
						accepted = true;
				}
				else
					accepted = true;
			}
			else
				canceled = true;
		} while (!accepted && !canceled);
		
		if (!canceled && isInputOK) {
			effectElement.setEffectName(ee.getConfigElement().getEffectName());
			
			for (int i = 0; i < settingTypes.length; i++) {
				String[] field = new String[2];
				field[0] = settingTypes[i];
				
				boolean found = false;
				for (int j = 0; j < content.getComponentCount() && !found; j++) {
					Component c = content.getComponent(j);
					
					if (c != null && c.getName() != null && c.getName().equals(settingTypes[i])) {
						if (c instanceof JSpinner)
							field[1] = Integer.toString((Integer)((JSpinner)c).getValue());
						else if (c instanceof IntField)
							field[1] = ((IntField)c).getText();
						else if (c instanceof FloatField)
							field[1] = ((FloatField)c).getText();
						else if (c instanceof JComboBox) {
							String selection = "";
							if (((JComboBox)c).getSelectedItem() instanceof ComboBoxColorItem) {
								ComboBoxColorItem cbci = (ComboBoxColorItem)((JComboBox)c).getSelectedItem();
								
								if (cbci.getStandardUrn() != null &&
										!cbci.getStandardUrn().equals("")) {
									selection = cbci.getStandardUrn();
								}
								else
									selection = cbci.getText();								
							}
							else {
								selection = (String)((JComboBox)c).getSelectedItem();
								selection = selection.trim();
								int pos = selection.indexOf("-");
								if (pos != -1) {
									selection = selection.substring(pos+1);
									selection = selection.trim();
								}
							}
								
							field[1] = selection;
						}
						else if (c instanceof JCheckBox) {
							if (((JCheckBox)c).isSelected())
								field[1] = Integer.toString(1);
							else
								field[1] = Integer.toString(0);
						}
						else if (c instanceof TimeField) {
							double time = HelperFunctions.timeStringToDouble(((TimeField)c).getText());
							field[1] = String.valueOf(time);
						}
						else if (c instanceof JTextField)
							field[1] = ((JTextField)c).getText();
						
						found = true;
					}
				}
				
				if (!effectElement.addAttribute(field)) {
					System.out.println("Problem during adding attribute: " + settingTypes[i]);
					return null;
				}					
			}
			
			effectElement.setStartTime(HelperFunctions.timeStringToDouble(fieldStartTime.getText()));	
			effectElement.setEndTime(HelperFunctions.timeStringToDouble(fieldEndTime.getText()));
			
			editingRect = "";
			
			return effectElement;
		}
		else {
			editingRect = "";
			isInputOK = true;
			lblMessage.setText("");
			return null;
		}
	}

	/**
	 * Method for handling keyboard events with key is pressed
	 * @param e KeyEvent to handle
	 */
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE)
			e.consume();
		
		lblMessage.setText(null);
	}

	/**
	 * Method for handling keyboard events with key is released
	 * @param e KeyEvent to handle
	 */
	public void keyReleased(KeyEvent e) {
		if (e.getSource() instanceof JFormattedTextField) {
			double startTime = 0.0;
			double endTime = 0.0;
			
			String name = ((JFormattedTextField)e.getSource()).getName();
			if (name.equals("startTime")) {
				startTime = HelperFunctions.timeStringToDouble(((JFormattedTextField)e.getSource()).getText());
				endTime = HelperFunctions.timeStringToDouble(fieldEndTime.getText());
			}
			else if (name.equals("endTime")) {
				startTime = HelperFunctions.timeStringToDouble(fieldStartTime.getText());
				endTime = HelperFunctions.timeStringToDouble(((JFormattedTextField)e.getSource()).getText());
			}
			
			isInputOK = validateTimeInput(startTime, endTime);
		}
		else if (e.getSource() instanceof EffectPanel) {
			if (e.getKeyCode() == KeyEvent.VK_DELETE) {
				BarElement toRemove = null;
				for (int i = 0; i < bar.size() && toRemove == null; i++) {
					BarElement be = bar.elementAt(i);
					ColorRectangle key = be.getColorRectangle();
					if (key.getColor() == selColor)
						toRemove = be;
				}
				
				if (toRemove != null)
					bar.remove(toRemove);
			}
			
			repaint();
		}
	}

	/**
	 * Method for handling keyboard events with key is pressed and released
	 * @param e KeyEvent to handle
	 */
	public void keyTyped(KeyEvent e) {
		if (e.getSource() instanceof JFormattedTextField) {
			String name = ((JFormattedTextField)e.getSource()).getName();
			if (name.equals("startTime") || name.equals("endTime")) {
				int position = ((JFormattedTextField)e.getSource()).getCaretPosition();
				int number = Character.getNumericValue(e.getKeyChar());
				switch (position) {
				case 3:
				case 6:
					// check if pressed key is bigger than 5
					if (number > 5) {
						e.consume();
						lblMessage.setText("Not allowed input(<6)!");
						isInputOK = false;
					}
					else
						isInputOK = true;
					break;
				}
			}
		}
	}

	/**
	 * Method for handling the actions performed
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("contextSettings")) {
			boolean found = false;
			for (int i = 0; i < bar.size() && !found; i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.contains(mouseContextPosX, HEIGHT/2)) {
					editingRect = key.getIdentifier();
					EffectElement ee = showEffectSettingsDialog(null, be.getEffectElement());
					if (ee != null) {
						be.setEffectElement(ee);
						key.x = (int)(ee.getStartTime()*70);
						key.width = (int)((ee.getEndTime()-ee.getStartTime())*70);
					}
					found = true;
					effectElement = new EffectElement(effectElement.getConfigElement());
				}
			}
			mouseContextPosX = -1;
		}
		else if (e.getActionCommand().equals("contextDelete")) {
			BarElement toRemove = null;
			for (int i = 0; i < bar.size() && toRemove == null; i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.contains(mouseContextPosX, HEIGHT/2))
					toRemove = be;
			}
			
			if (toRemove != null)
				bar.remove(toRemove);
			
			mouseContextPosX = -1;
		}
		else if (e.getActionCommand().equals("setToStartTime")) {
			fieldStartTime.setText("00:00:00.0");
			
			double startTime = HelperFunctions.timeStringToDouble(fieldStartTime.getText());
			double endTime = HelperFunctions.timeStringToDouble(fieldEndTime.getText());
						
			isInputOK = validateTimeInput(startTime, endTime);
		}
		else if (e.getActionCommand().equals("setToEndTime")) {
			double time = this.getWidth()/70.0;
			String strEndTime = HelperFunctions.doubleToTimeString(time);
			
			fieldEndTime.setText(strEndTime);
			
			double startTime = HelperFunctions.timeStringToDouble(fieldStartTime.getText());
			double endTime = HelperFunctions.timeStringToDouble(fieldEndTime.getText());
						
			isInputOK = validateTimeInput(startTime, endTime);
		}
		
		repaint();
	}
	
	/**
	 * Method for handling mouse click
	 * @param e MouseEvent to handle
	 */
	public void mouseClicked(MouseEvent e) {
		// check for double click
		if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() > 1) {
			boolean found = false;
			for (int i = 0; i < bar.size() && !found; i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.contains(e.getX(), HEIGHT/2)) {
					editingRect = key.getIdentifier();
					EffectElement ee = showEffectSettingsDialog(null, be.getEffectElement());
					if (ee != null) {
						be.setEffectElement(ee);
						key.x = (int)(ee.getStartTime()*70);
						key.width = (int)((ee.getEndTime()-ee.getStartTime())*70);
					}
					found = true;
					effectElement = new EffectElement(effectElement.getConfigElement());
				}
			}
		}
		else if (e.getButton() == MouseEvent.BUTTON1) {
			for (int i = 0; i < bar.size(); i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.contains(e.getX(), e.getY())) {
					key.setColor(selColor);
					selectedRect = (ColorRectangle)key.clone();
					this.requestFocus();
				}
				else
					key.setColor(color);
			}			
		}
		repaint();
	}

	/**
	 * Method for handling mouse entered
	 * @param e MouseEvent to handle
	 */
	public void mouseEntered(MouseEvent e) {}

	/**
	 * Method for handling mouse exited
	 * @param e MouseEvent to handle
	 */
	public void mouseExited(MouseEvent e) {	}

	/**
	 * Method for handling mouse pressed
	 * @param e MouseEvent to handle
	 */
	public void mousePressed(MouseEvent e) {		
		if (e.isPopupTrigger()) {
			mouseContextPosX = e.getX();
			contextMenu.show(e.getComponent(), e.getX(), e.getY());
		}
		
		if (e.getButton() == MouseEvent.BUTTON1) {
			for (int i = 0; i < bar.size(); i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle key = be.getColorRectangle();
				if (key.contains(e.getX(), e.getY())) {
					key.setColor(selColor);
					selectedRect = (ColorRectangle)key.clone();
				}
				else
					key.setColor(color);
			}
			oldDragPosX = e.getX();
			repaint();
			
			leftMouseButtonPressed = true;
		}
	}

	/**
	 * Method for handling mouse released
	 * @param e MouseEvent to handle
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.isPopupTrigger()) {
			mouseContextPosX = e.getX();
			contextMenu.show(e.getComponent(), e.getX(), e.getY());
		}
		
		if (selectedRect != null && oldDragPosX != -1 && e.getButton() == MouseEvent.BUTTON1) {
			boolean found = false;
			for (int i = 0; i < bar.size() && !found; i++) {
				BarElement be = bar.elementAt(i);
				ColorRectangle rc = be.getColorRectangle();
				if (rc.getIdentifier().equals(selectedRect.getIdentifier())) {
					rc.setColor(color);
					found = true;
				}
			}
			selectedRect = null;
			oldDragPosX = -1;
			leftMouseButtonPressed = false;
		}
	}
	
	public boolean isFocusTraversable() {   
        return true;
    }
	
	public boolean isFocusable() {   
        return true;
    }

	/**
	 * Method for handling mouse dragged
	 * @param e MouseEvent to handle
	 */
	public void mouseDragged(MouseEvent e) {
		if (selectedRect != null && leftMouseButtonPressed) { // && e.getButton() == MouseEvent.BUTTON1) {
			if (oldDragPosX != e.getX()) {
				boolean moveOverlapped = false;
				ColorRectangle tmpRect = (ColorRectangle)selectedRect.clone();
				int difference = e.getX()-oldDragPosX;
				if (oldDragPosX > e.getX())
					difference = (int)Math.ceil((difference)/7.0)*7;
				else if (oldDragPosX < e.getX())
					difference = (int)Math.floor((difference)/7.0)*7;
				tmpRect.x += difference;
				
				if (tmpRect.x >= 0 && (tmpRect.x + tmpRect.width) <= this.getWidth()) {
					//check for overlapping
					Iterator<BarElement> it = bar.iterator();
					while (it.hasNext()) {
						BarElement be = (BarElement) it.next();
						ColorRectangle key = be.getColorRectangle();
						if (!moveOverlapped && key.intersects(tmpRect) && !tmpRect.getIdentifier().equals(key.getIdentifier()))
							moveOverlapped = true;
					}
					
					if (!moveOverlapped) {
						boolean found = false;
						for (int i = 0; i < bar.size() && !found; i++) {
							BarElement be = bar.elementAt(i);
							ColorRectangle rc = be.getColorRectangle();
							if (rc.getIdentifier().equals(selectedRect.getIdentifier())) {
								// set new x position and new start and end time
								rc.x += difference;
								
								// every 7 pixel is 0.1 second
								EffectElement ee = be.getEffectElement();
								double startTime = rc.x/70.0; // get seconds			
								DecimalFormat df2 = new DecimalFormat("#########0.0");
								String value = df2.format(startTime);
								value = value.replace(",", ".");
								ee.setStartTime(new Double(value).doubleValue());
										
								double endTime = (rc.x+rc.width)/70.0; // get seconds	
								value = df2.format(endTime);
								value = value.replace(",", ".");
								ee.setEndTime(new Double(value).doubleValue());
								
								found = true;
							}
						}
						selectedRect.x += difference;
						
						if (difference != 0)
							oldDragPosX = e.getX();
					}
				}
				
				repaint();
			}
		}
	}

	/**
	 * Method for handling mouse moved
	 * @param e MouseEvent to handle
	 */
	public void mouseMoved(MouseEvent e) {}
	
	/**
	 * Method for clearing the complete bar
	 */
	public void clearBar() {
		bar = new Vector<BarElement>();
		repaint();
	}

	/**
	 * Method for retrieving the effect name
	 * @return String with name
	 */
	public String getNameOfEffect() {
		return nameOfEffect;
	}

	/**
	 * Method for setting the effect name
	 * @param nameOfEffect String with the name
	 */
	public void setNameOfEffect(String nameOfEffect) {
		this.nameOfEffect = nameOfEffect;
	}

	/**
	 * Method for retrieving the bars
	 * @return Vector<BarElement> with the bars
	 */
	public Vector<BarElement> getBars() {
		return bar;
	}
	
	/**
	 * Method for retrieving number of bars
	 * @return int number of bars
	 */
	public int getBarCount() {
		if (bar != null)
			return bar.size();
		else
			return 0;
	}
	
	/**
	 * Method for adding a bar
	 * @param bar BarElement to add
	 */
	public void addBar(BarElement bar) {
		if (bar != null) {
			this.bar.add(bar);
			repaint();
		}
	}
	
	/**
	 * Method for getting the color of the bar
	 * @return Color with the color
	 */
	public Color getColor() {
		return color;
	}
	
	/**
	 * Method for retrieving the config element
	 * @return ConfigElement with the config element
	 */
	public ConfigElement getConfigElement() {
		return configElement;
	}
	
	/**
	 * Class for providing color comboboxes
	 * @author Markus Waltl
	 */
	private class MyComboBoxEditor extends BasicComboBoxEditor {
		private Color color_ = null;
		
		public MyComboBoxEditor(Color color) {
			super();
			color_ = color;
		}
		
		public Component getEditorComponent(){
			JComponent comp = (JComponent) super.getEditorComponent();
			comp.setBorder(new EtchedBorder());
			comp.setBackground(color_);
			return comp;
		}
	}
	
	/**
	 * Class for providing the renderer for the combobox
	 * @author Markus Waltl
	 */
	private class ColorCellRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = -2123711306239796235L;

		private Border border;

		private JComboBox cb;

		ColorCellRenderer(JComboBox cb)
		{
			this.cb = cb;

			// Leave a 10-pixel separator between the flag icon and country name.

			setIconTextGap (10);

			// Swing labels default to being transparent; the container's color
			// shows through. To change a Swing label's background color, you must
			// first make the label opaque (by passing true to setOpaque()). Later,
			// you invoke setBackground(), passing the new color as the argument.

			setOpaque (true);

			// This border is placed around a cell that is selected and has focus.

			border = BorderFactory.createLineBorder (Color.RED, 1);
		}

		public Component getListCellRendererComponent (JList list,
				Object value,
				int index,
				boolean isSelected,
				boolean cellHasFocus)
		{
			ComboBoxColorItem c = (ComboBoxColorItem) value;
			setText(c.getText());
			
			if (c.getColor() != null)
				setIcon(new DiamondIcon(c.getColor()));

			if (isSelected)

			{
				setBackground (list.getSelectionBackground ());
				setForeground (list.getSelectionForeground ());
			}
			else
			{
				setBackground (list.getBackground ());
				setForeground (list.getForeground ());
			}

			setFont (list.getFont ());

			// list.isEnabled() always returns true for a combobox's list, even if
			// the combobox is disabled.

			setEnabled (cb != null ? cb.isEnabled () : list.isEnabled ());

			if (isSelected && cellHasFocus) // cellHasFocus ignored for JComboBox
				setBorder (border);
			else
				setBorder (null);

			return this;
		}
	}
	
	/**
	 * Method for validating the time
	 * @param startTime double with the start time of an effect
	 * @param endTime double with the end time of an effect
	 * @return true if the time is valid else false
	 */
	private boolean validateTimeInput(double startTime, double endTime) {
		if (startTime >= endTime || startTime < 0 || (endTime*70) > this.getWidth()) {
			lblMessage.setText("Check time!");
			return false;
		}
		
		ColorRectangle curRect = new ColorRectangle((int)(startTime*70), POSY, (int)((endTime-startTime)*70), HEIGHT, editingRect);
					
		boolean overlapped = false;
		
		for (int i = 0; i < bar.size() && !overlapped; i++) {
			BarElement be = bar.elementAt(i);
			ColorRectangle key = be.getColorRectangle();
			if (key.intersects(curRect) && !key.getIdentifier().equals(curRect.getIdentifier())) {
				lblMessage.setText("Overlapping!");
				overlapped = true;
				return false;
			}						
		}
		
		return true;
	}
}
