/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.awt.Color;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.text.DecimalFormat;

/**
 * Class providing helper functions
 * @author Markus Waltl
 */
public class HelperFunctions {
	/**
	 * Method for calculating a hex value to a Color object
	 * @param hex String with the hex value
	 * @return Color object with the color
	 */
	public static Color hexToRGB(String hex) {
		// input: #ffffff
		if (hex == null || hex.length() != 7)
			return Color.black;
		
		hex = hex.substring(1); // removing #
		
		int r = Integer.parseInt(hex.substring(0, 2), 16);
		int g = Integer.parseInt(hex.substring(2, 4), 16);
		int b = Integer.parseInt(hex.substring(4, 6), 16);
		
		return new Color(r, g, b);
	}
	
	/**
	 * Method for transforming a Color object into a hex string 
	 * @param rgb Color object with the color
	 * @return String with the hex value
	 */
	public static String rgbToHex(Color rgb) {
		if (rgb == null)
			return null;
		
		String s= "00000"+Integer.toHexString(rgb.getRGB());
		return s.substring(s.length()-6);
	}
	
	/**
	 * Method for converting a double value to a time string
	 * @param time variable with the time
	 * @return String with the time string formatted as HH:MM:SS.s
	 */
	public static String doubleToTimeString(double time) {
		int hours = 0;
		int minutes = 0;
		double seconds = time;
		minutes = (int) seconds / 60;
		seconds = seconds % 60;
		hours = minutes / 60;
		minutes = minutes % 60;
		
		DecimalFormat df = new DecimalFormat("#########0.0");
		String value = df.format(seconds);
		value = value.replace(",", ".");
		seconds = new Double(value).doubleValue();
		
		String timeString = "";
		if (hours < 10)
			timeString = "0" + hours + ":";
		else
			timeString = hours + ":";

		if (minutes < 10)
			timeString += "0" + minutes + ":";
		else
			timeString += minutes + ":";

		if (seconds < 10)
			timeString += "0" + seconds;
		else
			timeString += seconds;
		
		return timeString;
	}
	
	/**
	 * Method for converting a time string to a double value
	 * @param timeString variable with the time string formatted as HH:MM:SS.s
	 * @return double with the time as double value
	 */
	public static double timeStringToDouble(String timeString) {
		double retVal = 0.0;
		String[] split = timeString.split(":");
		retVal = Double.valueOf(split[0])*3600;
		retVal += Double.valueOf(split[1])*60;
		retVal += Double.valueOf(split[2]);
		return retVal;
	}
	
	/**
	 * Method for rounding a value to a defined precision
	 * @param value double with the value to be rounded
	 * @param precision int with the precision to be rounded to
	 * @return BigDecimal with the rounded value
	 */
	public static BigDecimal round(double value, int precision) {
		return round(new BigDecimal(value), precision);
	}
	
	/**
	 * Method for rounding a value to a defined precision
	 * @param value BigDecimal with the value to be rounded
	 * @param precision int with the precision to be rounded to
	 * @return BigDecimal with the rounded value
	 */
	public static BigDecimal round(BigDecimal value, int precision) {
		BigDecimal result = null;
	    MathContext context = new MathContext((value.precision() - value.scale() + precision), RoundingMode.HALF_UP);
	    try {
	      result = value.round(context);
	    } catch( ArithmeticException ae ) {
	      ae.printStackTrace();
	    }
		
		return result;
	}
}
