/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package lib;

import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.*;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;

import app.SEVinoView;

/**
 * @author Markus Waltl
 * thanks to Mathias Lux (mathias@juggle.at) for his advice
 * 
 * KNOWN ISSUES:
 * -------------
 * Color comparison has problems with some videos
 */
public class ShotDetection {
	private SEVinoView gui_ = null;	
	
	private String ffmpegCmd = "";
    private int videoHeight;
    private int videoWidth;

    private double fps = -1.0;
    private int modul = 1;
    private int totalFrames;
    
    private double duration_ = 0.0;
    private String durationString_ = "";
    private String format_ = "";
    private String decoder_ = "";
    private String bitrate_ = "";
    private String dimension_ = "";
    private Hashtable<Double, BufferedImage> images_ = new Hashtable<Double, BufferedImage>();
    
    private long red_prev = -1;
    private long green_prev = -1;
    private long blue_prev = -1;
    private long red_prev_prev = -1;
    private long green_prev_prev = -1;
    private long blue_prev_prev = -1;
    private double threshold = 20.0;
    private final static double MIN_GLOBAL_THRESHOLD = 15.0;
    private int frameCounter_ = 0;
    private final static int FRAME_IDLE = 20;
    
    private boolean isStopped = false;
    
    private InputStream in = null;
    private InputStream error = null;
    
    /**
     * Constructor
     * @param file String with path and name for creating shot detection
     * @param frameSkip int with the number of frames to skip
     * @param gui SEVinoView with the GUI
     * @throws IOException
     */
    public ShotDetection(String file, int frameSkip, SEVinoView gui) throws IOException {
    	gui_ = gui;
    	isStopped = false;
    	
    	String OS = System.getProperty("os.name").toLowerCase();
    	if (OS.indexOf("windows") > -1)
    		ffmpegCmd = "3rd\\tools\\ffmpeg.exe";
    	else
    		ffmpegCmd = "ffmpeg";
    	
        if (frameSkip != 0)
        	modul = frameSkip;
        
        // call ffmpeg for info:
        //System.out.println("Getting info from file.");
        Process process = null;
        process = Runtime.getRuntime().exec(new String[] {ffmpegCmd, "-i", file});
        
        error = process.getErrorStream();
        StringBuilder sb = new StringBuilder(512);
        int tmp = -1;
        while ((tmp = error.read()) > -1) {
            sb.append((char) tmp);
        }
                
        String duration = "";
        String video = "";
        
        if (OS.indexOf("windows") > -1) {
        	duration = sb.substring(sb.indexOf("Duration: ") + "Duration: ".length(), sb.indexOf("\n", sb.indexOf("Duration: ")) - 1);
        	video = sb.substring(sb.indexOf("Video: ") + "Video: ".length(), sb.indexOf("\n", sb.indexOf("Video: ")) - 1);
        }
        else {
        	duration = sb.substring(sb.indexOf("Duration: ") + "Duration: ".length(), sb.indexOf("\n", sb.indexOf("Duration: ")));
        	video = sb.substring(sb.indexOf("Video: ") + "Video: ".length(), sb.indexOf("\n", sb.indexOf("Video: ")));
        }
        
        System.out.println("video = " + video);
        System.out.println("duration = " + duration);
        
        StringTokenizer st = new StringTokenizer(duration, ",");
        durationString_ = st.nextToken().trim();
        String[] dur = durationString_.split(":");
        duration_ = ((Double.valueOf(dur[0])*60)+Double.valueOf(dur[1]))*60+Double.valueOf(dur[2]); 
        @SuppressWarnings("unused")
		String starttime = st.nextToken().trim();
        
        String bitrate = st.nextToken().trim();
        int pos = bitrate.indexOf("bitrate:");
        if (pos != -1)
        	bitrate = bitrate.substring(9).trim();
        	
        bitrate_ = bitrate; 
        
        st = new StringTokenizer(video, ",");
        String decoder = st.nextToken().trim();
        String format = st.nextToken().trim();
        decoder_ = decoder;
        format_ = format;
        String size = st.nextToken().trim();
        
        while (st.hasMoreTokens()) {
	        String strFPS = st.nextToken().trim();
	        pos = strFPS.indexOf("tb");
	        if (pos != -1 && fps == -1.0)
	        	fps = Double.valueOf(strFPS.substring(0, pos));
	        else {
	        	pos = strFPS.indexOf("fps");
	        	if (pos != -1)
	        		fps = Double.valueOf(strFPS.substring(0, pos));
	        }
        }
        
        dimension_ = size;
        
        pos = size.indexOf("[");
        if (pos != -1)
        	size = size.substring(0, pos);
        size = size.trim();
        String[] sizeArray = size.split("x");

        st = new StringTokenizer(duration, ",");
        String time = st.nextToken();
        String[] timeBits = time.split("\\:");

        totalFrames = (int) ((Integer.parseInt(timeBits[0]) * 60 * 60 + Integer.parseInt(timeBits[1]) * 60) * fps);
        totalFrames += Integer.parseInt(timeBits[2].split("\\.")[0]) * fps;
        totalFrames += Integer.parseInt(timeBits[2].split("\\.")[1]);

        videoWidth = Integer.parseInt(sizeArray[0]);
        videoHeight = Integer.parseInt(sizeArray[1]);

        process = Runtime.getRuntime().exec(new String[] {ffmpegCmd, "-i", file, "-deinterlace", "-f", "rawvideo",
        		"-pix_fmt", "yuv420p", "-"});
        
        in = process.getInputStream();
        error = process.getErrorStream();
    }
    
    public void startDetection() {
    	new Thread(new ErrorReaderThread(error)).start();
    	shotDetection(in);
    }

    /**
     * Method for building the shot detection
     * @param in InputStream with video for which to build shot detection
     */
    private void shotDetection(InputStream in) {
    	// needed if odd video size
    	if (videoWidth % 2 != 0)
    		videoWidth = videoWidth - 1;
    	
    	if (videoHeight % 2 != 0)
    		videoHeight = videoHeight - 1;
    	
        int pixels = videoWidth * videoHeight;
        
        byte uu, yy, vv;
        int[] rgb = new int[3];
        byte[] buffer = new byte[pixels + pixels / 2];
        //long time = 0;
        int frameCount = 0;

        // use a 10 MB buffer:
        BufferedInputStream bin = new BufferedInputStream(in, 10 * 104 * 1024);
        try {
            boolean isReadable = true;
            while (isReadable && !isStopped) {
                isReadable = readNextFrame(buffer, bin) > -1;
                // the next frame has been read.
                if (frameCount % modul == 0) { // take only every x-th frame
                	BufferedImage frame = new BufferedImage(videoWidth, videoHeight, BufferedImage.TYPE_INT_RGB);
                	// use RGB comparison            	               	
                	long nred = 0, ngreen = 0, nblue = 0;
                	
                	for (int x = 0; x < videoWidth; x++) {
                        for (int y = 0; y < videoHeight; y++) {
                            yy = (buffer[y * videoWidth + x]);
                            uu = (buffer[pixels + (y >> 1) * (videoWidth >> 1) + (x >> 1)]);
                            vv = (buffer[pixels + (pixels >> 2) + (y >> 1) * (videoWidth >> 1) + (x >> 1)]);
                            yuv2rgb(toInt(yy), toInt(uu), toInt(vv), rgb);
                            // create a picture ...
                            frame.getRaster().setPixel(x, y, rgb);
                            
                            nred += rgb[0];
                            ngreen += rgb[1];
                            nblue += rgb[2];
                        }
                    }
                    
                    // Calculate relative difference between current and previous frame.
                	if (red_prev == -1 && green_prev == -1 && blue_prev == -1) {
                		// first frame should always be added!
                		DecimalFormat df2 = new DecimalFormat("#########0.0");
                    	String value = df2.format((frameCount/fps));
                    	value = value.replace(",", ".");
            			double time = new Double(value).doubleValue();
            			
            			Image scaledImage = frame.getScaledInstance(60, 45, BufferedImage.SCALE_FAST);
            	 		frame = new BufferedImage(60, 45, BufferedImage.TYPE_INT_RGB);
            	 		frame.getGraphics().drawImage(scaledImage, 0, 0, null);
                    	images_.put(time, frame);
                	}
                	else {
	                    long nred_diff = 0, ngreen_diff = 0, nblue_diff = 0;
	                    double red_ch = 0.0, green_ch = 0.0, blue_ch = 0.0;
	                    double redp_ch = 0.0, greenp_ch = 0.0, bluep_ch = 0.0;
	
	                    nred_diff = Math.abs(red_prev - nred);
	                    if (red_prev != 0) {
	                        red_ch = (nred_diff * 100.0) / red_prev;
	                    }
	
	                    ngreen_diff = Math.abs(green_prev - ngreen);
	                    if (green_prev!= 0)
	                        green_ch = (ngreen_diff * 100.0) / green_prev;
	
	
	                    nblue_diff = Math.abs(blue_prev - nblue);
	                    if (blue_prev != 0)
	                        blue_ch = (nblue_diff * 100.0) / blue_prev;
	                    
	                    
	                    nred_diff = Math.abs(red_prev_prev - nred);
	                    if (red_prev_prev != 0) {
	                        redp_ch = (nred_diff * 100.0) / red_prev_prev;
	                    }
	
	                    ngreen_diff = Math.abs(green_prev_prev - ngreen);
	                    if (green_prev_prev != 0)
	                        greenp_ch = (ngreen_diff * 100.0) / green_prev_prev;
	
	
	                    nblue_diff = Math.abs((blue_prev_prev + blue_prev)/2 - nblue);
	                    if ((blue_prev_prev + blue_prev)/2 != 0)
	                        bluep_ch = (nblue_diff * 100.0) / ((blue_prev_prev + blue_prev)/2);
	
	                    if ((red_ch > threshold && green_ch > threshold && blue_ch > threshold) &&
	                    		(redp_ch > threshold || greenp_ch > threshold || bluep_ch > threshold)) {
	                    	frameCounter_ = 0;
	                    	DecimalFormat df2 = new DecimalFormat("#########0.0");
	                    	String value = df2.format((frameCount/fps));
	                    	value = value.replace(",", ".");
	            			double time = new Double(value).doubleValue();
	            			
	            			Image scaledImage = frame.getScaledInstance(60, 45, BufferedImage.SCALE_FAST);
	            	 		frame = new BufferedImage(60, 45, BufferedImage.TYPE_INT_RGB);
	            	 		frame.getGraphics().drawImage(scaledImage, 0, 0, null);
	                    	images_.put(time, frame);
	                    	
	                    	//System.out.println("Added: " + r_ch + " -- " + g_ch + " -- " + b_ch + " | " + rp_ch + " -- " + gp_ch + " -- " + bp_ch);
	                    	
	                    	double val = Math.min(red_ch, green_ch);
	                    	val = Math.min(val, blue_ch);
	                    	double val2 = Math.min(redp_ch, greenp_ch);
	                    	val2 = Math.min(val2, bluep_ch);
	                    	
	                    	double tmp_threshold = (Math.min(val, val2))/2;
	                    	if (tmp_threshold >= MIN_GLOBAL_THRESHOLD && tmp_threshold <= threshold*2)     		
	                    		threshold = tmp_threshold;
	                    	else if (tmp_threshold > threshold*2)
	                    		threshold = threshold*2;
	                    	/*else if (tmp_threshold < MIN_GLOBAL_THRESHOLD)
	                    		threshold =  MIN_GLOBAL_THRESHOLD;*/
	                    	
	                    	//System.out.println("New threshold: " + threshold);
	                    }
	                    else {
	                    	frameCounter_++;
	                    	
	                    	if (frameCounter_ % FRAME_IDLE == 0) {
	                    		threshold = threshold * 0.75;
	                    		frameCounter_ = 0;
	                    	}	
	                    }
	                    //System.out.println(r_ch + " -- " + g_ch + " -- " + b_ch + " | " + rp_ch + " -- " + gp_ch + " -- " + bp_ch);
                	}
                	red_prev_prev = red_prev;
                	green_prev_prev = green_prev;
                	blue_prev_prev = blue_prev;
                    red_prev = nred;
                    green_prev = ngreen;
                    blue_prev = nblue;
                }
                frameCount++;
                if (frameCount % (int) fps == 0) {
                    gui_.updateProgressBar((frameCount*100)/totalFrames);                	
                }
            }
            
            //System.out.println("Processed 100%");
          	gui_.updateProgressBar(100);
          	
          	Runtime r = Runtime.getRuntime();
          	r.gc();
          	
          	if (in != null) {
          		in.close();
          		in = null;
          	}
          	
          	if (error != null) {
          		error.close();
          		error = null;
          	}
          	
        } catch (IOException ioe) {
        	ioe.printStackTrace();
        }
    }

    /**
     * Method for reading the next frame
     * @param buffer byte[] with the buffer
     * @param in InputStream with the video
     * @return int with bytes read else -1
     * @throws IOException
     */
    private int readNextFrame(byte[] buffer, InputStream in) throws IOException {
        int bytesRead = in.read(buffer);
        if (bytesRead < 0) {
            return -1;
        }
        while ((bytesRead < buffer.length)) {
            int i = in.read(buffer, bytesRead, buffer.length - bytesRead);
            if (i < 0) {
                return -1;
            }
            bytesRead += i;
        }
        return bytesRead;
    }

    /**
     * Method for convert byte to int
     * @param b byte to convert
     * @return int representation of byte
     */
    private int toInt(byte b) {
        return b & 0xff;
    }

    /**
     * Method to convert YUV to RGB
     * @param y int with Y
     * @param u int with U
     * @param v int with V
     * @param rgb int[] with RGB array for output
     * @return int[] with RGB array
     */
    private int[] yuv2rgb(int y, int u, int v, int[] rgb) {
        int c = y - 16;
        int d = u - 128;
        int e = v - 128;
        rgb[0] = (298 * c + 409 * e + 128) >> 8;
        rgb[1] = (298 * c - 100 * d - 208 * e + 128) >> 8;
        rgb[2] = (298 * c + 516 * d + 128) >> 8;

        // clamp:
        for (int i = 0; i < rgb.length; i++) {
            rgb[i] = Math.min(rgb[i], 255);
            rgb[i] = Math.max(rgb[i], 0);
        }
        return rgb;
    }

    /**
     * Method for retrieving the FPS
     * @return double with FPS
     */
    public double getFPS() {
    	return fps;
    }
    
    /**
     * Method for retrieving the duration
     * @return double with duration
     */
    public double getDuration() {
    	return duration_;
    }
    
    /**
     * Method for retrieving the duration as string
     * @return String with duration
     */
    public String getDurationString() {
    	return durationString_;
    }
    
    /**
     * Method for retrieving the video format
     * @return String with format
     */
    public String getFormat() {
    	return format_;
    }
    
    /**
     * Method for retrieving the decoder
     * @return String with decoder
     */
    public String getDecoder() {
    	return decoder_;
    }
    
    /**
     * Method for retrieving the dimension
     * @return String with dimension
     */
    public String getDimension() {
    	return dimension_;
    }
    
    /**
     * Method for retrieving the bitrate
     * @return String with bitrate
     */
    public String getBitrate() {
    	return bitrate_;
    }
    
    /**
     * Method for retrieving the shots
     * @return Hashtable<Double, BufferedImage> with shots for every found time
     */
    public Hashtable<Double, BufferedImage> getShots() {
    	return images_;
    }
    
    /**
     * Stops the shot detection
     */
    public synchronized void stopDetection() {
    	isStopped = true;
    }
}