/*
 * This file is part of SEVino.
 *
 * SEVino is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SEVino is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SEVino.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2008, 2009, 2010, 2011 Alpen-Adria-Universitaet Klagenfurt, Markus Waltl.
 */
package xml;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.transform.stream.StreamSource;

import org.iso.mpeg.mpeg7.AgentType;
import org.iso.mpeg.mpeg7.ControlledTermUseType;
import org.iso.mpeg.mpeg7.CreationToolType;
import org.iso.mpeg.mpeg7.CreatorType;
import org.iso.mpeg.mpegv.sedl.DescriptionMetadataType;
import org.iso.mpeg.mpegv.sedl.EffectBaseType;
import org.iso.mpeg.mpegv.sedl.GroupOfEffects;
import org.iso.mpeg.mpeg7.OrganizationType;
import org.iso.mpeg.mpegv.sedl.SEM;
import org.iso.mpeg.mpegv.sedl.SEMBaseType;
import org.iso.mpeg.mpeg7.TermUseType;
import org.iso.mpeg.mpeg7.TextualType;

import lib.AuthorInfo;
import lib.EffectElement;
import lib.SEMDefinitions;

/**
 * SEMParser
 * 
 * Class for parsing a sensory effects metadata description
 * 
 * @author Markus Waltl (Alpen-Adria-Universitaet Klagenfurt)
 * @version 1.0
 */
public class SEMParser {
	private double curTime = -1;
	private long timeScale = 1;
	private AuthorInfo authorInfo = null;
	private Vector<EffectElement> effectElements = null;
	private EffectElement tmpElement = null;
		
	/**
	 * Constructor
	 */
	public SEMParser() {
		effectElements = new Vector<EffectElement>();
	}
	
	/**
	 * Method for parsing a SEM description
	 * @param fileToParse
	 * @return Object[] with parsed data ([0] = Author information, [1] = Effects and GroupOfEffects
	 * @throws JAXBException - if an error occured.
	 * @throws IOException - if an IO error occured.
	 */
	public Object[] parse(String fileToParse) throws JAXBException, IOException {
		JAXBContext jaxbContext;
		try {
			File file = new File(fileToParse);
			if (!file.exists() || !file.canRead())
				throw new IOException("IO error: file does not exist or cannot be read!");
			
			jaxbContext = JAXBContext.newInstance(SEM.class);
			Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
			JAXBElement<SEM> jSem = unmarshaller.unmarshal(new StreamSource(file), SEM.class);
			
			SEM sem = jSem.getValue();
			
			// extract time information
			Map<QName, String> semAttributes = sem.getOtherAttributes();
			if (semAttributes != null) {
				for (QName key : semAttributes.keySet()) {
					if (key.getLocalPart().equals(SEMDefinitions.TIMESCALE)) {
						timeScale = Long.valueOf(semAttributes.get(key));
					}
				}
			}
			
			// extract author information
			DescriptionMetadataType dmt = sem.getDescriptionMetadata();
			if (dmt != null) {
				authorInfo = new AuthorInfo();
				authorInfo.setConfidence(dmt.getConfidence());
				authorInfo.setCreationTime(dmt.getCreationTime());
				authorInfo.setLastUpdate(dmt.getLastUpdate());
				authorInfo.setVersion(dmt.getVersion());
								
				if (dmt.getComment() != null && dmt.getComment().getFreeTextAnnotationsAndStructuredAnnotationsAndDependencyStructures() != null) {
					List<Object> o = dmt.getComment().getFreeTextAnnotationsAndStructuredAnnotationsAndDependencyStructures();
					if (o != null) {
						if (o.get(0) instanceof TextualType) {
							TextualType tt = (TextualType)o.get(0);
							authorInfo.setCreatorName(tt.getValue());
						}
					}
				}
				
				List<CreatorType> lc = dmt.getCreators();
				if (lc != null) {
					CreatorType ct = lc.get(0);
					ControlledTermUseType ctut = ct.getRole();
					List<org.iso.mpeg.mpeg7.InlineTermDefinitionType.Name> names = ctut.getNames();
					if (names != null) {
						org.iso.mpeg.mpeg7.InlineTermDefinitionType.Name name = names.get(0);
						authorInfo.setCreatorName(name.getValue());
					}
					
					AgentType at = ct.getAgent();
					if (at != null && at instanceof OrganizationType) {
						OrganizationType ot = (OrganizationType)at;
						List<org.iso.mpeg.mpeg7.OrganizationType.Name> oNames = ot.getNames();
						if (oNames != null) {
							org.iso.mpeg.mpeg7.OrganizationType.Name name = oNames.get(0);
							authorInfo.setCreatorOrganization(name.getValue());
						}
					}
					
					List<CreationToolType> instruments = ct.getInstruments();
					if (instruments != null) {
						boolean found = false;
						for (int i = 0; i < instruments.size() && !found; i++) {
							CreationToolType instrument = instruments.get(i);
							if (instrument.getTool() != null) {
								TermUseType tut = instrument.getTool();
								List<org.iso.mpeg.mpeg7.InlineTermDefinitionType.Name> tNames = tut.getNames();
								if (tNames != null) {
									org.iso.mpeg.mpeg7.InlineTermDefinitionType.Name name = tNames.get(0);
									authorInfo.setInstrument(name.getValue());
									found = true;
								}								
							}
						}
					}
				}
			}
			
			// extract effects and groupofeffects
			List<SEMBaseType> semContent = sem.getDeclarationsAndGroupOfEffectsAndEffects();
			if (semContent != null) {
				for (int i = 0; i < semContent.size(); i++) {
					SEMBaseType sbt = semContent.get(i);
					if (sbt instanceof EffectBaseType) {
						handleEffect((EffectBaseType)sbt);						
						curTime = -1;
					}
					else if (sbt instanceof GroupOfEffects) {
						GroupOfEffects goe = (GroupOfEffects)sbt;
										
						Map<QName, String> goeAttributes = goe.getOtherAttributes();
						if (goeAttributes != null) {
							for (QName key : goeAttributes.keySet()) {
								if (key.getLocalPart().equals(SEMDefinitions.PTS)) {
									double lVal = Double.valueOf(goeAttributes.get(key));
									curTime = lVal/timeScale;
								}
								else if (key.getLocalPart().equals(SEMDefinitions.ANCHORELEMENT)) {
									// currently not needed
								}
							}
						}
								
						List<SEMBaseType> groupEffects = goe.getEffectsAndReferenceEffects();
						if (groupEffects != null) {
							for (SEMBaseType goeSBT : groupEffects) {
								if (goeSBT instanceof EffectBaseType) {
									handleEffect((EffectBaseType)goeSBT);
								}
								// reference effect not supported (for now!)
							}
						}
						
						curTime = -1;
					}
				}
			}			
		} catch (JAXBException e) {
			throw new JAXBException("Parsing error: " + e.getMessage());
		} catch (Exception e) {
			throw new JAXBException("Unexpected error: " + e.getMessage());
		}		
		
		return getSEMContent();
	}

	/**
	 * Method for retrieving the SEM content
	 * @return Object[] with the content (0 = AuthorInfo, 1 = Vector<EffectElement>)
	 */
	public Object[] getSEMContent() {
		if (effectElements.isEmpty())
			return null;
		
		Object[] o = new Object[2];
		o[0] = authorInfo;
		o[1] = effectElements;
		return o;
	}
	
	/**
	 * Method for handling effects, e.g., extracting effect attributes
	 * @param ebt EffectBaseType with the effect to be handled
	 */
	private void handleEffect(EffectBaseType ebt) {
		tmpElement = new EffectElement();
		tmpElement.setEffectName(ebt.getClass().getSimpleName());
		
		Map<QName, String> effectAttributes = ebt.getOtherAttributes();
		if (effectAttributes != null) {
			for (QName key : effectAttributes.keySet()) {
				if (key.getLocalPart().equals(SEMDefinitions.PTS)) {
					double lVal = Double.valueOf(effectAttributes.get(key));
					curTime = lVal/timeScale;
				}
				else if (key.getLocalPart().equals(SEMDefinitions.ANCHORELEMENT)) {
					// not interesting for now within effects
				}
				else {
					String[] att = new String[2];
					att[0] = key.getLocalPart();
					att[1] = effectAttributes.get(key);
					tmpElement.addAttribute(att);
				}
			}
		}
		
		// copy effectbasetype attributes
		if (ebt.getAlt() != null && !ebt.getAlt().equals(""))
			tmpElement.addAttribute(new String[] { SEMDefinitions.ALT, ebt.getAlt() });
		
		if (ebt.getAutoExtraction() != null && ebt.getAutoExtraction().value() != null &&
				!ebt.getAutoExtraction().value().equals(""))
			tmpElement.addAttribute(new String[] { SEMDefinitions.AUTOEXTRACTION, ebt.getAutoExtraction().value() });
		
		if (ebt.getDuration() != null)
			tmpElement.addAttribute(new String[] { SEMDefinitions.DURATION, String.valueOf(ebt.getDuration()) });
		
		if (ebt.getFade() != null) {
			double time = ebt.getFade().doubleValue()/timeScale;
			tmpElement.addAttribute(new String[] { SEMDefinitions.FADE, String.valueOf(time) });
		}
		
		if (ebt.getLocation() != null && !ebt.getLocation().equals(""))
			tmpElement.addAttribute(new String[] { SEMDefinitions.LOCATION, ebt.getLocation() });
		
		if (ebt.getPriority() != null)
			tmpElement.addAttribute(new String[] { SEMDefinitions.PRIORITY, String.valueOf(ebt.getPriority()) });
		
		if (ebt.isActivate()) {
			boolean found = false;
			for (int j = 0; j < effectElements.size() && !found; j++) {
				EffectElement ee = effectElements.elementAt(j);
				if (ee.getEffectName() != null && ee.getEffectName().equals(tmpElement.getEffectName())) {
					if (ee.getEndTime() == -1.0) {
						ee.setEndTime(curTime);
						found = true;
					}
				}
			}
			
			tmpElement.setStartTime(curTime);
			effectElements.add(tmpElement);
		}
		else {
			boolean found = false;
			for (int j = 0; j < effectElements.size() && !found; j++) {
				EffectElement ee = effectElements.elementAt(j);
				if (ee.getEffectName() != null && ee.getEffectName().equals(tmpElement.getEffectName())) {
					if (ee.getEndTime() == -1.0) {
						ee.setEndTime(curTime);
						found = true;
					}
				}
			}
		}
		
		tmpElement = null;
	}
}